/* NOTE!!! This script will be collapsed to single line! Do not forget ; and don't
   use single line comments! */
(function (supportAjax, scriptId) {
    var OBJECT_NAME = "YandexApplicationsAPIBackendImpl";

    if (window.YandexApplicationsAPIBackend && window.YandexApplicationsAPIBackend.__appImpl) {
      /* Already loaded */
      return;
    }

    var call_func_impl;
    if (supportAjax) {
      var houston = (function() {
        var obj = window[OBJECT_NAME];
        if (!obj) {
          throw new Error("Failed to get object");
        }

        obj.onBound(String(scriptId));

        var nestingLevel = 0;
        var sessionId = 0;

        return {
          call: function(name, argument_array) {
            nestingLevel++;
            try {
              var sendObj = {
                type: "call",
                function_name: name,
                argument_array: argument_array
              };
              while (true) {
                var responseStr = obj.onSyncMessage(String(sessionId), String(nestingLevel), JSON.stringify(sendObj));
                var responseObj = JSON.parse(responseStr);
                if (responseObj.type == "response") {
                  if (responseObj.exception_message) {
                    throw new Error(responseObj.exception_message);
                  } else {
                    return responseObj.value;
                  }
                } else {
                  var fun = window.eval(responseObj.function_name);
                  var fun_exception = null;
                  var fun_res = null;
                  var call_args;
                  if (responseObj.unjson_args) {
                    call_args = Array.prototype.map.call(responseObj.argument_array, function (item) { return JSON.parse(item); });
                  } else {
                    call_args = responseObj.argument_array;
                  }
                  try {
                      fun_res = fun.apply(null, call_args);
                  } catch (e) {
                      fun_exception = e;
                  }
                  sendObj = {
                    type: "response",
                    value: JSON.stringify(fun_res),
                    exception_message: fun_exception == null ? null : String(fun_exception)
                  };
                }
              }
            } finally {
              nestingLevel--;
              if (nestingLevel == 0) {
                sessionId++;
              }
            }
          }
        };
      })();

      call_func_impl = houston.call;

    } else {
        call_func_impl = function(name, argument_array) {
            if (!window[OBJECT_NAME]) {
                console.log("API ERROR: Can't find API implementation with name [" + OBJECT_NAME + "]");
                return;
            }
            var func = window[OBJECT_NAME][name];
            if (!func) {
                console.log("API ERROR: Unknown method name " + OBJECT_NAME + ".[" + name + "]");
                return;
            }
            return func.apply(window[OBJECT_NAME], argument_array);
        };
    }

    function call_func() {
        if (arguments.length == 0) {
            console.log("API ERROR: Method name missed");
            return;
        }
        var name = arguments[0];
        var args = Array.prototype.splice.call(arguments, 1);
        /* we don't want any 'undefined' bullshit in Java code */
        args = Array.prototype.map.call(args, function (item) { return item === undefined ? null : item; });
        return call_func_impl(name, args);
    };

    window.YandexApplicationsAPIBackend = {
      on : function (event, handler) {
          call_func("onRegisterHandler", event, handler);
      },

      openVertService : function (service, url) {
          call_func("onLinkCallback", service, url);
      },

      setQueryText : function (text) {
          call_func("onQueryChanged", text);
      },

      log : function (logs) {
          call_func("log", logs);
      },

      readyForShow : function () {
          call_func("onReadyForShow");
      },

      voiceAnswer : function (json) {
          call_func("voiceAnswer", json);
      },

      releasePointer : function () {
          call_func("releasePointer");
      },

      capturePointer : function () {
          call_func("capturePointer");
      },

      isOurTabActive : function () {
          return call_func("isOurTabActive");
      },

      openFullScreen : function () {
        return call_func("openFullScreen");
      },

      closeFullScreen : function () {
        return call_func("closeFullScreen");
      },

      isPageVisible : function () {
          return call_func("isPageVisible");
      },

      openRelatedQuery : function(requestDataJson) {
          return call_func("openRelatedQuery", requestDataJson);
      },

      __appImpl: true /* A tag showing this is a true implementation */
    };

    if (supportAjax) {
      window.YandexApplicationsAPIBackend.onAjaxSuccess = function(id, url) {
          call_func("onAjaxSuccess", id, url);
      };
      window.YandexApplicationsAPIBackend.onAjaxError = function(id, url, message, details) {
          call_func("onAjaxError", id, url, message, details);
      };
      window.YandexApplicationsAPIBackend.onAjaxMeta = function(id, url, metaInfo) {
          call_func("onAjaxMeta", id, url, metaInfo);
      };
      window.YandexApplicationsAPIBackend.onAjaxReady = function(url) {
          call_func("onAjaxReady", url);
      };
      window.YandexApplicationsAPIBackend.onAjaxUnload = function(url) {
          call_func("onAjaxUnload", url);
      };
      window.YandexApplicationsAPIBackend.ajaxUpdateTabs = function(id, url, metaInfo) {
          call_func("ajaxUpdateTabs", id, url, metaInfo);
      };
      /* deprecated */
      window.YandexApplicationsAPIBackend.changeLocation = function(url) {
          call_func("ajaxChangeLocation", url);
      };
      window.YandexApplicationsAPIBackend.ajaxChangeLocation = function(url) {
          call_func("ajaxChangeLocation", url);
      };
      window.YandexApplicationsAPIBackend.callHouston = function() {
          call_func("ping");
      };
    }

    window.open = function (url, windowName, windowFeatures) {
        call_func("onLinkCallback", "", url);
    };
})