var Y76815dc1_cb9e_4a65_8c3a_9b137d57d62e = {
    win: typeof window != 'undefined' && window,
    doc: typeof document != 'undefined' && document,
    displayPropName: 'display',

    viewportH: function (docElem, win)
    {
        var a = docElem['clientHeight'], b = win['innerHeight'];
        return Math.max(a,b);
    },

    viewportW: function (docElem, win) {
        var a = docElem['clientWidth'], b = win['innerWidth'];
        return Math.max(a,b);
    },

    getCssDisplay: function (el) {
        if (el.style[this.displayPropName]) {
            return el.style[this.displayPropName];
        }

        if (el.tagName == "SCRIPT") {
            return "none";
        }

        var s = el.currentStyle || el.ownerDocument.defaultView.getComputedStyle(el, null);
        if (!s[this.displayPropName]) {
            return "LI,P,TR".indexOf(el.tagName) > -1 ? "block" : null;
        }
        if (s[this.displayPropName] == "block" && el.tagName == "TD") {
            return "feaux-inline";
        }
        return s[this.displayPropName];
    },

    infoFromElement: function (el) {
        el = el && !el.nodeType ? el[0] : el;
        if (!el || 1 !== el.nodeType) {
            return false;
        }
        var s = this.getCssDisplay(el);
        if (s == "none") {
            return false;
        }

        var clientRect = el.getBoundingClientRect();
        if (clientRect.width <= 0 || clientRect.height <= 0) {
            return null;
        }

        return {
            display: s,
            bounds:  clientRect
        };
    },

    rectContainsPoint: function (rect, x, y) {
        return x >= rect.left
               && x <= (rect.left + rect.width)
               && y >= rect.top
               && y <= (rect.top + rect.height)
    },

    rectContainsRect: function (outer, inner) {
        var bottom = inner.top + inner.height;
        var right = inner.left + inner.width;
        return this.rectContainsPoint(outer, inner.left, inner.top)
               && this.rectContainsPoint(outer, inner.left, bottom)
               && this.rectContainsPoint(outer, right, inner.top)
               && this.rectContainsPoint(outer, right, bottom);
    },

    rectUnion: function (rect1, rect2) {
        var left = Math.min(rect1.left, rect2.left);
        var top = Math.min(rect1.top, rect2.top);

        var bottom = Math.max(rect1.top + rect1.height, rect2.top + rect2.height);
        var right = Math.max(rect1.left + rect1.width, rect2.left + rect2.width);

        return this.makeRect(left, top, right - left, bottom - top);
    },

    rectIntersectsWithRect: function (rect1, rect2) {
        var bottom = rect2.top + rect2.height;
        var right = rect2.left + rect2.width;
        return !(right < rect1.left
                 || rect2.left > rect1.left + rect1.width
                 || bottom < rect1.top
                 || rect2.top > rect1.top + rect1.height);
    },

    makeRect: function (x, y, w, h) {
        return {
            left: x,
            top: y,
            width: w,
            height: h
        };
    },

    normalize: function (a) {
        // clean up double line breaks and spaces
        if (!a) {
            return '';
        }
        return a.trim()
            .replace(/ +/g, " ")
            .replace(/[\t]+/gm, "");
    },



    prepareTree: function(src) {
        var joined = [];
        var result = [];
        var rootCount = src.length;

        for(var i = 0; i < rootCount; ++i) {
            var info = src[i];
            if (joined.indexOf(info) >= 0)
                continue;

            var parent = {
                children: [],
                info: {
                    node: info.node.parentNode,
                    contains: info.contains,
                    bounds: info.bounds,
                    text: ''
                }
            };

            var child =
            {
                info: info,
                children: []
            };

            parent.children.push(child);

            for (var j = i+1; j < rootCount; ++j) {
                var target = src[j];
                if (info.node.parentNode != target.node.parentNode)
                    continue;

                parent.children.push(
                    {
                        children: [],
                        info: target
                    }
                );
                joined.push(target);

            }
            result.push(parent);
        }

        return result;
    },

    containsNode: function (container, contained) {
        var parent = contained.parentNode;
        for (var distance = 1; parent; ++distance) {
            if (parent == container)
                return distance;
            parent = parent.parentNode;
        }
        return 0;
    },

    buildTree: function (roots) {
        var _this = this;
        var joined = [];
        roots.forEach(function (contained) {
            var parents = roots.map(function (candidate) {
                if (candidate == contained)
                    return null;

                var candidateInfo = candidate.info;
                var containedInfo = contained.info;

                var d = _this.containsNode(candidateInfo.node, containedInfo.node);
                if (d == 0)
                    return null;

                if (!_this.rectIntersectsWithRect(candidateInfo.bounds, containedInfo.bounds))
                    return null;

                return {
                    distance: d,
                    node: candidate
                };
            }).filter(function (candidate) {
                return candidate;
            }).sort(function (a, b) {
                return a.distance - b.distance;
            });

            if (parents.length) {
                var parent = parents[0].node;
                parent.children.push(contained);
                joined.push(contained);
            }
        });

        return roots.filter(function (node) {
            return joined.indexOf(node) < 0;
        });
    },

    findByNode: function(all, nodeToFind) {
        var allLen = all.length;
        for (var i = 0; i < allLen; ++i) {
            var toTest = all[i];
            if (toTest.node == nodeToFind)
                return toTest;
        }

        return null;
    },

    plainRoot: function (root, all) {
        var info = root.info;

        var bounds = info.bounds;
        var text = info.text;
        var node;
        var walk = this.doc.createTreeWalker(info.node, NodeFilter.SHOW_TEXT, null, false);
        while (node = walk.nextNode()) {
            var found = this.findByNode(all, node);
            if (!found)
                continue;

            text += ' ' + found.text;
            bounds = this.rectUnion(bounds, found.bounds);
        }

        return {
            info: {
                node: info.node,
                contains: info.contains,
                bounds: bounds,
                text: this.normalize(text)
            },
            children: []
        };
    },

    getLeaves: function(root) {
        var childCount = root.children.length;
        if (!childCount)
            return [root.info];

        var result = [];
        for (var i = 0; i < childCount; ++i) {
            var child = root.children[i];
            var childLeaves = this.getLeaves(child);
            result = result.concat(childLeaves);
        }
        return result;
    },

    plainTree: function (tree) {
        var _this = this;
        return tree.map(function (node) {
            var all = _this.getLeaves(node);
            var info = _this.plainRoot(node, all).info;
            return {
                txt: info.text,
                t: info.bounds.top,
                l: info.bounds.left,
                w: info.bounds.width,
                h: info.bounds.height,
                in: info.contains,
                tg: info.node.parentNode.tagName
            }
        })
    },

    getTexts: function () {
        var docElem = this.doc && this.doc.documentElement;
        var vp = this.makeRect(0, 0, this.viewportW(docElem, this.win), this.viewportH(docElem, this.win));
        var all = [];

        var node;
        var walk = this.doc.createTreeWalker(this.doc.body, NodeFilter.SHOW_TEXT, null, false);
        while (node = walk.nextNode()) {

            var plainText = this.normalize(node.nodeValue);
            if (!plainText)
                continue;

            var parent = node.parentNode;
            var info = this.infoFromElement(parent);

            if (!info)
                continue;

            var inViewport = this.rectIntersectsWithRect(vp, info.bounds);
            var containsViewport = this.rectContainsRect(info.bounds, vp);

            if (!inViewport)
                continue;

            all.push({
                         node: node,
                         contains: containsViewport,
                         bounds: info.bounds,
                         text: plainText
                     });
        }

        var tree = this.prepareTree(all);
        tree = this.buildTree(tree);
        var plain = this.plainTree(tree);
        return {
            h: vp.height,
            w: vp.width,
            all: plain
        };
    }
};

